using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using EPSCOMLib;

namespace EPS2Lenses
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
        private System.Windows.Forms.PictureBox pictureBox1;
        private IContainer components;
        private EPSLens[] UserLens;
        private EPSLensRegion[] UserRegion;		
		private EPSEnvironment env;		
        private EPSVec2 vecIn1 = new EPSVec2Class();
		private EPSVec2 vecOut1 = new EPSVec2Class();
		private EPSVec2 vecIn2 = new EPSVec2Class();
		private EPSVec2 vecOut2 = new EPSVec2Class();

        private GsiDemo.MultiModalMultiUserIntegrator multiModalMultiUserIntegrator1;               
		private Graphics gc;

        private int[] UserTouching;

        //stuff specifically needed for color blending
        private Color[] UserColors = { Color.Tomato, Color.GreenYellow, Color.LightBlue, Color.LightYellow };
        private EPSLens[][] ColorLenses;
        private EPSLensRegion[][] ColorRegions;
        private EPSEnvironment[] ColorEnv;
        private int TotalUsers = 4;
        private const int BaseColor = 64;
        private const int RED = 0;
        private const int GREEN = 1;
        private const int BLUE = 2;
        private const int TOTALCOLORS = 3;
        

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            this.pictureBox1 = new System.Windows.Forms.PictureBox();
            this.multiModalMultiUserIntegrator1 = new GsiDemo.MultiModalMultiUserIntegrator();
            ((System.ComponentModel.ISupportInitialize)(this.pictureBox1)).BeginInit();
            this.SuspendLayout();
            // 
            // pictureBox1
            // 
            this.pictureBox1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pictureBox1.Location = new System.Drawing.Point(0, 0);
            this.pictureBox1.Name = "pictureBox1";
            this.pictureBox1.Size = new System.Drawing.Size(694, 551);
            this.pictureBox1.TabIndex = 0;
            this.pictureBox1.TabStop = false;
            this.pictureBox1.MouseMove += new System.Windows.Forms.MouseEventHandler(this.pictureBox1_MouseMove);
            this.pictureBox1.Paint += new System.Windows.Forms.PaintEventHandler(this.pictureBox1_Paint);
            // 
            // multiModalMultiUserIntegrator1
            // 
            this.multiModalMultiUserIntegrator1.BackColor = System.Drawing.Color.MidnightBlue;
            this.multiModalMultiUserIntegrator1.Location = new System.Drawing.Point(175, 453);
            this.multiModalMultiUserIntegrator1.Name = "multiModalMultiUserIntegrator1";
            this.multiModalMultiUserIntegrator1.Size = new System.Drawing.Size(136, 96);
            this.multiModalMultiUserIntegrator1.TabIndex = 1;
            this.multiModalMultiUserIntegrator1.PostureMove += new GestureEngine.Postures.PostureEventHandler(this.multiModalMultiUserIntegrator1_PostureMove);
            this.multiModalMultiUserIntegrator1.PostureDown += new GestureEngine.Postures.PostureEventHandler(this.multiModalMultiUserIntegrator1_PostureDown);
            this.multiModalMultiUserIntegrator1.PostureUp += new GestureEngine.Postures.PostureEventHandler(this.multiModalMultiUserIntegrator1_PostureUp);
            // 
            // Form1
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(694, 551);
            this.Controls.Add(this.multiModalMultiUserIntegrator1);
            this.Controls.Add(this.pictureBox1);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
            this.Name = "Form1";
            this.Text = "Form1";
            this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
            this.Load += new System.EventHandler(this.Form1_Load);
            ((System.ComponentModel.ISupportInitialize)(this.pictureBox1)).EndInit();
            this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		//Initialize EPS so it has a Lens of a particular style
		private void SetupEPS () 
		{
			// Create a new lenses 
            UserLens = new EPSLens[TotalUsers];
            UserRegion = new EPSLensRegion[TotalUsers];
            env = new EPSEnvironmentClass();

            //Color
            ColorEnv = new EPSEnvironment[TOTALCOLORS];
            
            for (int i = 0; i < TOTALCOLORS; ++i)
            {
                ColorEnv[i] = new EPSEnvironmentClass();
            }
            ColorLenses = new EPSLens[TotalUsers][];
            ColorRegions = new EPSLensRegion[TotalUsers][];
            Random random = new Random();
            
            for (int i = 0; i < TotalUsers; ++i)
            {
                UserLens[i] = new EPSLensClass();
                UserLens[i].RemoveAllRegions();
                UserRegion[i] = new EPSLensRegionClass();
                UserLens[i].AddRegion(UserRegion[i]);                
                env.AddLens(UserLens[i]);

                ColorLenses[i] = new EPSLens[TOTALCOLORS];
                ColorRegions[i] = new EPSLensRegion[TOTALCOLORS];
                //random start position
                EPSVec2 startPos = new EPSVec2Class();
                startPos.x = random.Next(this.Width);
                startPos.y = random.Next(this.Height);
                UserLens[i].Center = startPos;

                
                
                for (int j = 0; j < TOTALCOLORS; ++j)
                {
                    ColorLenses[i][j] = new EPSLensClass();
                    ColorLenses[i][j].RemoveAllRegions();
                    ColorRegions[i][j] = new EPSLensRegionClass();                    
                    ColorLenses[i][j].AddRegion(ColorRegions[i][j]);
                    ColorEnv[j].AddLens(ColorLenses[i][j]);
                    ColorLenses[i][j].Center = startPos;
                    ColorLenses[i][j].BlendMode = epsBlendModes.epsBlend_None;
                }
                SetLensColors(i);                                                

            }									            
            this.BackColor = Color.DarkSlateGray;
            pictureBox1.Visible = true;
		}

        private void SetLensColors(int UserId)
        {
            ColorLenses[UserId][RED].Magnification = (float) UserColors[UserId].R / (float) BaseColor;
            ColorLenses[UserId][GREEN].Magnification = (float)UserColors[UserId].G / (float)BaseColor;
            ColorLenses[UserId][BLUE].Magnification = (float)UserColors[UserId].B / (float)BaseColor;
        }
        
        private void multiModalMultiUserIntegrator1_SpeechRecognition(int UserId, int SpeechIndex)        
        {
            //play an audio cue
            System.Media.SoundPlayer sp = new System.Media.SoundPlayer(@"c:\windows\media\Windows XP Information Bar.wav");
            sp.Play();
            switch (SpeechIndex)
            {
                case 0: //Square Shape
                    SetLensShape(UserId, epsFocusShapes.epsShape_Square);
                    break;
                case 1: //circle shape
                    SetLensShape(UserId, epsFocusShapes.epsShape_Circle);
                    break;
                case 2: //point shape
                    SetLensShape(UserId, epsFocusShapes.epsShape_Point);
                    break;
                case 3: //small mag
                    SetLensMagnification(UserId, 1.5f);
                    break;
                case 4: //mid mag
                    SetLensMagnification(UserId, 2.0f);
                    break;
                case 5: //large mag
                    SetLensMagnification(UserId, 2.5f);
                    break;
                case 6: //small size
                    SetLensSize(UserId, 50);
                    break;
                case 7: //mid size
                    SetLensSize(UserId, 100);
                    break;
                case 8: //large size
                    SetLensSize(UserId, 200);
                    break;
                case 9: //red color
                    SetUserColor(UserId, Color.Red);
                    break;
                case 10: //blue color
                    SetUserColor(UserId, Color.Blue);
                    break;
                case 11: //Yellow color
                    SetUserColor(UserId, Color.Goldenrod);
                    break;
                case 12: //Purple color
                    SetUserColor(UserId, Color.Fuchsia);
                    break;
                
            }
            
        }

        private bool MultiModalUserSelect(ref int UserId)
        {
            if (UserId < TotalUsers && UserId >= 0)
            {
                if (UserTouching[UserId] < 0)
                {
                    for (int i = 0; i < TotalUsers; ++i)
                    {
                        if (UserTouching[i] > 0)
                        {
                            UserId = i;
                            break;
                        }
                    }
                }                                    
                return true;
            }
            UserId = -1;
            return false;
        }
    

        private void SetUserColor(int UserId, Color color)
        {
            if (MultiModalUserSelect(ref UserId))
            {
                UserColors[UserId] = color;
                SetLensColors(UserId);
                pictureBox1.Invalidate();
            }

        }

        private void SetLensSize(int UserId, int Size)
        {
            if (MultiModalUserSelect(ref UserId))
            {                
                UserRegion[UserId].DropoffWidth = Size;
                for (int i = 0; i < TOTALCOLORS; ++i)
                {
                    ColorRegions[UserId][i].DropoffWidth = Size;
                }
                pictureBox1.Invalidate();
            }
        }
   

        private void SetLensShape(int UserId, epsFocusShapes Shape)
        {
            if (MultiModalUserSelect(ref UserId))
            {                
                UserLens[UserId].Shape = Shape;
                for (int i = 0; i < TOTALCOLORS; ++i)
                {
                    ColorLenses[UserId][i].Shape = Shape;
                }
                pictureBox1.Invalidate();
            }
        }

        private void SetLensMagnification(int UserId, float MagnificationAmount)
        {
            if (MultiModalUserSelect(ref UserId))
            {                
                UserLens[UserId].Magnification = MagnificationAmount;                
                pictureBox1.Invalidate();
            }
        }

        void multiModalMultiUserIntegrator1_Started(object sender, EventArgs e)
        {
            multiModalMultiUserIntegrator1.SetSpeechCommands("Square Shape,Circle Shape,Point Shape,Small Mag,Medium Mag,Large Mag,Small Size,Medium Size,Large Size,Red Color,Blue Color,Yellow Color,Purple Color,");
            TotalUsers = multiModalMultiUserIntegrator1.GetTotalGestureClients();
            UserTouching = new int[TotalUsers];
            for (int i = 0; i < TotalUsers; ++i)
            {
                UserTouching[i] = -1;
            }
            SetupEPS();
        }

        // Convert a vector into an integer point
        private Point V2pt(EPSVec2 vector)
		{
			return (new Point ( (int) vector.x , (int) vector.y) );
		}

		private void Form1_Load(object sender, System.EventArgs e)
		{
            pictureBox1.Visible = false;
            multiModalMultiUserIntegrator1.Started += new EventHandler(multiModalMultiUserIntegrator1_Started);
            multiModalMultiUserIntegrator1.SpeechRecognition += new GsiDemo.SpeechRecognitionEventHandler(multiModalMultiUserIntegrator1_SpeechRecognition);
            multiModalMultiUserIntegrator1.Start();
		}

		private void pictureBox1_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			int gridsize = 30;
			gc = e.Graphics;
			int xincr = pictureBox1.Width / gridsize;
			int yincr = pictureBox1.Height / gridsize;
			int lastx = 0;

			for (int y = 0; y <= pictureBox1.Height; y += yincr ) 
			{
				for (int x = 0; x <= pictureBox1.Width; x += xincr ) 
				{
					vecIn1.set(lastx,y);
					vecIn2.set(x,y);

					// use the method in env to calculate the output
					env.get_MagnifiedPoint2DandMagnification(vecIn1, vecOut1);
                    env.get_MagnifiedPoint2DandMagnification(vecIn2, vecOut2);
                    /*ColorEnv[RED].get_MagnifiedPoint2DandMagnification(vecIn1, vecOut1);
                    ColorEnv[RED].get_MagnifiedPoint2DandMagnification(vecIn2, vecOut2);*/


					Point p1 = V2pt (vecOut1);
                    Point p2 = V2pt(vecOut2);

					//p1.Offset (-1, -1);
                    //p2.Offset(, -1);
                    

                    gc.DrawLine(Pens.Black, p1, p2);
                    float magAmount = env.get_MagnificationAmount(vecIn1);
                    //float magAmount = ColorEnv[RED].get_MagnificationAmount(vecIn1);

                    float HalfBaseColor = (float) BaseColor;
                    int RedValue = (int) ((float) HalfBaseColor * ColorEnv[RED].get_MagnificationAmount(vecIn1));
                    int GreenValue = (int)((float)HalfBaseColor * ColorEnv[GREEN].get_MagnificationAmount(vecIn1));
                    int BlueValue = (int)((float)HalfBaseColor * ColorEnv[BLUE].get_MagnificationAmount(vecIn1));                                        

                    Color FillColor = Color.FromArgb(255, (RedValue>255||RedValue<0)?255:RedValue, (GreenValue>255||GreenValue<0)?255:GreenValue, (BlueValue>255||BlueValue<0)?255:BlueValue);

                    gc.FillRectangle(new SolidBrush(FillColor), p1.X, p1.Y, 10f * magAmount * magAmount, 10f * magAmount * magAmount);

					lastx = x;
				}
				lastx = 0;
			}


		}

		private void pictureBox1_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
			{
				EPSVec2 focalCenter = new EPSVec2Class();
				focalCenter.set(e.X, e.Y);                
				UserLens[0].Center = focalCenter;				
                for (int i = 0; i < TOTALCOLORS; ++i)
                {
                    ColorLenses[0][i].Center = focalCenter;
                }
                pictureBox1.Invalidate();
			}
			else if (e.Button == MouseButtons.Right)
			{
				EPSVec2 focalCenter = new EPSVec2Class();
				focalCenter.set(e.X, e.Y);
				UserLens[1].Center = focalCenter;
                for (int i = 0; i < TOTALCOLORS; ++i)
                {
                    ColorLenses[1][i].Center = focalCenter;
                }
                pictureBox1.Invalidate ();
			}
			else
				return;

		}

        private void multiModalMultiUserIntegrator1_PostureDown(int PostureDetected, AxDIAMONDTOUCHLib._DDiamondTouchEvents_TouchEvent e)
        {
            UserTouching[e.receiverId] = 1;
        }

        private void multiModalMultiUserIntegrator1_PostureMove(int PostureDetected, AxDIAMONDTOUCHLib._DDiamondTouchEvents_TouchEvent e)
        {
            Point CenterPoint = new Point((e.right + e.left) / 2, (e.top + e.bottom) / 2);            
            if (e.receiverId < TotalUsers)
            {
                EPSVec2 focalCenter = new EPSVec2Class();
                focalCenter.set(CenterPoint.X, CenterPoint.Y);                
                UserLens[e.receiverId].Center = focalCenter;
                for (int i = 0; i < TOTALCOLORS; ++i)
                {
                    ColorLenses[e.receiverId][i].Center = focalCenter;
                }
                pictureBox1.Invalidate();
            }            
        }

        private void multiModalMultiUserIntegrator1_PostureUp(int PostureDetected, AxDIAMONDTOUCHLib._DDiamondTouchEvents_TouchEvent e)
        {
            UserTouching[e.receiverId] = -1;
        }

	}
}
